@@ -10,9 +10,17 @@
 #endif
 
 #include <string.h>
+#include <gio/gio.h>
 
 #include "soup-content-sniffer-stream.h"
-#include "soup.h"
+#include "soup-content-sniffer.h"
+#include "soup-message.h"
+
+static void soup_content_sniffer_stream_pollable_init (GPollableInputStreamInterface *pollable_interface, gpointer interface_data);
+
+G_DEFINE_TYPE_WITH_CODE (SoupContentSnifferStream, soup_content_sniffer_stream, G_TYPE_FILTER_INPUT_STREAM,
+			 G_IMPLEMENT_INTERFACE (G_TYPE_POLLABLE_INPUT_STREAM,
+						soup_content_sniffer_stream_pollable_init))
 
 enum {
 	PROP_0,
@@ -34,24 +42,23 @@ struct _SoupContentSnifferStreamPrivate {
 	GHashTable *sniffed_params;
 };
 
-static void soup_content_sniffer_stream_pollable_init (GPollableInputStreamInterface *pollable_interface, gpointer interface_data);
-
-G_DEFINE_TYPE_WITH_CODE (SoupContentSnifferStream, soup_content_sniffer_stream, G_TYPE_FILTER_INPUT_STREAM,
-                         G_ADD_PRIVATE (SoupContentSnifferStream)
-			 G_IMPLEMENT_INTERFACE (G_TYPE_POLLABLE_INPUT_STREAM,
-						soup_content_sniffer_stream_pollable_init))
-
 static void
 soup_content_sniffer_stream_finalize (GObject *object)
 {
 	SoupContentSnifferStream *sniffer = SOUP_CONTENT_SNIFFER_STREAM (object);
 
-	g_clear_object (&sniffer->priv->sniffer);
-	g_clear_object (&sniffer->priv->msg);
-	g_free (sniffer->priv->buffer);
-	g_clear_error (&sniffer->priv->error);
-	g_free (sniffer->priv->sniffed_type);
-	g_clear_pointer (&sniffer->priv->sniffed_params, g_hash_table_unref);
+	if (sniffer->priv->sniffer)
+		g_object_unref (sniffer->priv->sniffer);
+	if (sniffer->priv->msg)
+		g_object_unref (sniffer->priv->msg);
+	if (sniffer->priv->buffer)
+		g_free (sniffer->priv->buffer);
+	if (sniffer->priv->error)
+		g_error_free (sniffer->priv->error);
+	if (sniffer->priv->sniffed_type)
+		g_free (sniffer->priv->sniffed_type);
+	if (sniffer->priv->sniffed_params)
+		g_hash_table_unref (sniffer->priv->sniffed_params);
 
 	G_OBJECT_CLASS (soup_content_sniffer_stream_parent_class)->finalize (object);
 }
@@ -167,8 +174,7 @@ read_internal (GInputStream  *stream,
 
 	if (sniffer->priv->buffer) {
 		nread = MIN (count, sniffer->priv->buffer_nread);
-		if (buffer)
-			memcpy (buffer, sniffer->priv->buffer, nread);
+		memcpy (buffer, sniffer->priv->buffer, nread);
 		if (nread == sniffer->priv->buffer_nread) {
 			g_free (sniffer->priv->buffer);
 			sniffer->priv->buffer = NULL;
@@ -234,16 +240,6 @@ soup_content_sniffer_stream_skip (GInputStream  *stream,
 	return nskipped;
 }
 
-static gboolean
-soup_content_sniffer_stream_can_poll (GPollableInputStream *pollable)
-{
-	GInputStream *base_stream = G_FILTER_INPUT_STREAM (pollable)->base_stream;
-
-	return G_IS_POLLABLE_INPUT_STREAM (base_stream) &&
-		g_pollable_input_stream_can_poll (G_POLLABLE_INPUT_STREAM (base_stream));
-}
-
-
 static gboolean
 soup_content_sniffer_stream_is_readable (GPollableInputStream *stream)
 {
@@ -290,7 +286,9 @@ soup_content_sniffer_stream_create_source (GPollableInputStream *stream,
 static void
 soup_content_sniffer_stream_init (SoupContentSnifferStream *sniffer)
 {
-	sniffer->priv = soup_content_sniffer_stream_get_instance_private (sniffer);
+	sniffer->priv = G_TYPE_INSTANCE_GET_PRIVATE (sniffer,
+						     SOUP_TYPE_CONTENT_SNIFFER_STREAM,
+						     SoupContentSnifferStreamPrivate);
 	sniffer->priv->sniffing = TRUE;
 }
 
@@ -301,6 +299,8 @@ soup_content_sniffer_stream_class_init (SoupContentSnifferStreamClass *sniffer_c
 	GInputStreamClass *input_stream_class =
 		G_INPUT_STREAM_CLASS (sniffer_class);
  
+	g_type_class_add_private (sniffer_class, sizeof (SoupContentSnifferStreamPrivate));
+
 	object_class->finalize = soup_content_sniffer_stream_finalize;
 	object_class->set_property = soup_content_sniffer_stream_set_property;
 	object_class->get_property = soup_content_sniffer_stream_get_property;
@@ -314,26 +314,37 @@ soup_content_sniffer_stream_class_init (SoupContentSnifferStreamClass *sniffer_c
 				     "Sniffer",
 				     "The stream's SoupContentSniffer",
 				     SOUP_TYPE_CONTENT_SNIFFER,
-				     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
+				     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (
 		object_class, PROP_MESSAGE,
 		g_param_spec_object ("message",
 				     "Message",
 				     "The stream's SoupMessage",
 				     SOUP_TYPE_MESSAGE,
-				     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
+				     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 }
 
 static void
 soup_content_sniffer_stream_pollable_init (GPollableInputStreamInterface *pollable_interface,
 					   gpointer                       interface_data)
 {
-	pollable_interface->can_poll = soup_content_sniffer_stream_can_poll;
 	pollable_interface->is_readable = soup_content_sniffer_stream_is_readable;
 	pollable_interface->read_nonblocking = soup_content_sniffer_stream_read_nonblocking;
 	pollable_interface->create_source = soup_content_sniffer_stream_create_source;
 }
 
+GInputStream *
+soup_content_sniffer_stream_new (SoupContentSniffer *sniffer,
+				 SoupMessage        *msg,
+				 GInputStream       *base_stream)
+{
+	return g_object_new (SOUP_TYPE_CONTENT_SNIFFER_STREAM,
+			     "base-stream", base_stream,
+			     "message", msg,
+			     "sniffer", sniffer,
+			     NULL);
+}
+
 gboolean
 soup_content_sniffer_stream_is_ready (SoupContentSnifferStream  *sniffer,
 				      gboolean                   blocking,
