@@ -18,8 +18,12 @@ void  soup_socket_handshake_async (SoupSocket         *sock,
 				   SoupSocketCallback  callback,
 				   gpointer            user_data);
 
-GSocket   *soup_socket_get_gsocket  (SoupSocket *sock);
-GIOStream *soup_socket_get_iostream (SoupSocket *sock);
+GSocket   *soup_socket_get_gsocket    (SoupSocket *sock);
+GIOStream *soup_socket_get_connection (SoupSocket *sock);
+GIOStream *soup_socket_get_iostream   (SoupSocket *sock);
 
+#define SOUP_SOCKET_CLEAN_DISPOSE "clean-dispose"
+#define SOUP_SOCKET_USE_PROXY     "use-proxy"
+SoupURI *soup_socket_get_http_proxy_uri (SoupSocket *sock);
 
 #endif /* SOUP_URI_PRIVATE_H */
